# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.colors import ListedColormap


def plot_decision_regions(X, y, classifier, test_idx=None, resolution=0.02):
    # setup marker generator and color map
    markers = ('s', 'x', 'o', '^', 'v')
    colors = ('red', 'blue', 'lightgreen', 'gray', 'cyan')
    cmap = ListedColormap(colors[:len(np.unique(y))])
    # plot the decision surface
    x1_min, x1_max = X[:, 0].min() - 1, X[:, 0].max() + 1
    x2_min, x2_max = X[:, 1].min() - 1, X[:, 1].max() + 1
    xx1, xx2 = np.meshgrid(np.arange(x1_min, x1_max, resolution),
                           np.arange(x2_min, x2_max, resolution))
    Z = classifier.predict(np.array([xx1.ravel(), xx2.ravel()]).T)
    Z = Z.reshape(xx1.shape)
    plt.contourf(xx1, xx2, Z, alpha=0.4, cmap=cmap)
    plt.xlim(xx1.min(), xx1.max())
    plt.ylim(xx2.min(), xx2.max())
    # plot class samples
    for idx, cl in enumerate(np.unique(y)):
        plt.scatter(x=X[y == cl, 0], y=X[y == cl, 1],
                    alpha=0.8, c=cmap(idx), marker=markers[idx], label=cl)
    # highlight test samples
    if test_idx:
        X_test, y_test = X[test_idx, :], y[test_idx]
        plt.scatter(X_test[:, 0], X_test[:, 1], c='', alpha=1.0, linewidth=1,
                    marker='o', s=55, label='test set')



# -*- coding: utf-8 -*-
"""
————————————————————————————————————————————————————————————————————————————————————————
https://www.oreilly.com/learning/a-new-visz-to-beautifully-explore-correlations
太阳系相关性图

————————————————————————————————————————————————————————————————————————————————————————
"""

import math
import matplotlib.pyplot as plt
import numpy as np
import csv


def get_angles(orbit, number_of_datapoints, ax):
    start = (2 * math.pi) / 11 * (orbit - 1)
    stop = start + (math.pi / 2)
    if orbit > 0:
        ax.text(0, orbit - 0.1, "{0:.1f}".format(1 - float(orbit) / 10), verticalalignment="top",
                horizontalalignment="center", color="lightgray")
    return np.linspace(start, stop, number_of_datapoints, endpoint=True)


def get_y(angle, radius):
    return math.sin(angle) * radius


def get_x(angle, radius):
    return math.cos(angle) * radius


def label_to_idx(labels, label):
    center_idx_bool = labels == label
    return np.asscalar(np.where(center_idx_bool)[0]), center_idx_bool


def transform_to_correlation_dist(data):
    """"""
    y_corr = np.corrcoef(data.T)
    # we just need the magnitude of the correlation and don't care
    # whether it's positive or not
    abs_corr = np.abs(y_corr)
    return np.nan_to_num(abs_corr)


def transform_to_positive_corrs(data, sun_idx):
    y_corr = np.corrcoef(data.T)
    positive = y_corr[sun_idx]
    positive = positive >= 0
    return positive


def solar_corr(data, labels, center, orbits=10, show_window=True,
               image_path="solar.png", save_png=False):
    """"""
    labels = np.array(labels)
    center_idx, center_idx_bool = label_to_idx(labels, center)

    all_idx = np.logical_not(center_idx_bool)
    positive = transform_to_positive_corrs(data, center_idx)
    corr_dist = transform_to_correlation_dist(data)
    sun_corr_dist = corr_dist[center_idx]
    colors = np.linspace(0, 1, num=len(sun_corr_dist))

    step = 1.0 / orbits
    last_orbit = 0.0
    fig = plt.gcf()
    fig.set_size_inches(20, 20)
    labels_idx = np.array([center_idx])

    color_map = plt.get_cmap("Paired")
    color_map_circle = plt.get_cmap("Accent")

    ax = fig.add_subplot(111)
    ax.xaxis.set_visible(False)
    ax.yaxis.set_visible(False)
    ax.set_position([0.01, 0.01, 0.98, 0.98])  # set a new position

    # place sun:
    plt.scatter(0, 0, color=color_map(colors[center_idx]), s=600, label=labels[center_idx])
    ax.text(0.2, 0.2, str(labels[center_idx]), verticalalignment="bottom", horizontalalignment='left', color="gray")

    for orbit in range(1, orbits + 1):
        new_orbit = step * orbit + 0.1
        idx = (sun_corr_dist >= (1 - last_orbit)) & (sun_corr_dist > (1 - new_orbit)) & all_idx
        idx_int = np.where(idx)[0]

        corr_dists = []
        for index in idx_int:
            corr_dists = np.append(corr_dists, corr_dist[center_idx][index])
        sort_args = np.argsort(corr_dists)
        idx_int = idx_int[sort_args]

        labels_idx = np.append(labels_idx, idx_int)
        planets = sum(idx)
        angles = get_angles(orbit, planets, ax)

        # place planets
        while np.any(idx):
            remaining = sum(idx)
            current_planet = planets - remaining
            current_idx = idx_int[current_planet]
            angle = angles[current_planet]
            x = get_x(angle, orbit)
            y = get_y(angle, orbit)
            # current_idx = idx_int[current_planet]
            color = colors[current_idx]
            plt.scatter(x, y, color=color_map(color), s=250, label=labels[current_idx])
            planet_idx = current_idx
            idx[planet_idx] = False
            all_idx[planet_idx] = False

            planet_corr = corr_dist[planet_idx]
            # ax.text(x-0.35, y+0.2, "{0:.3f}".format(planet_corr[center_idx]))
            col = "#03C03C" if positive[planet_idx] else "#FF6961"
            if orbit == orbits:
                col = "grey"
            ax.text(x + 0.15, y + 0.15, str(labels[planet_idx]), verticalalignment="bottom", horizontalalignment='left',
                    color=col)
            moon_idx = (planet_corr >= 0.8) & all_idx
            moon_idx_int = np.where(moon_idx)[0]
            moons = sum(moon_idx)
            moon_angles = get_angles(0, moons, ax)

            # add orbit around planet if it has moons
            if np.any(moon_idx):
                moon_orbit = 0.5
                circle = plt.Circle((x, y), moon_orbit, color='lightgrey', alpha=0.8, fill=True, zorder=0)
                fig.gca().add_artist(circle)

            while np.any(moon_idx):
                remaining_moons = sum(moon_idx)
                current_moon = moons - remaining_moons
                current_moon_idx = moon_idx_int[current_moon]
                angle = moon_angles[current_moon]
                m_x = get_x(angle, moon_orbit) + x
                m_y = get_y(angle, moon_orbit) + y
                color = colors[current_moon_idx]
                plt.scatter(m_x, m_y, color=color_map(color), s=100, label=labels[current_moon_idx])
                col = "#03C03C" if positive[current_moon_idx] else "#FF6961"
                if orbit == orbits:
                    col = "grey"
                ax.text(m_x + 0.15, m_y + 0.05, str(labels[current_moon_idx]), verticalalignment="bottom",
                        horizontalalignment='left', color=col)
                moon_idx[current_moon_idx] = False
                idx[current_moon_idx] = False
                all_idx[current_moon_idx] = False

        last_orbit = new_orbit

        circle = plt.Circle((0, 0), orbit, color=color_map_circle(1 - ((orbit - 1) * step)), fill=False, zorder=0)
        fig.gca().add_artist(circle)

    labels_pos = np.array(labels)[labels_idx]
    recs = []
    ax = plt.gca()

    ax.axis("equal")
    plt.axis([-10, 10, -10, 10])
    if save_png:
        plt.savefig(image_path)

    if show_window:
        plt.show()


def main():
    input_csv = r'.\DataSet\jedi.csv'
    data = np.genfromtxt(input_csv, delimiter=",", skip_header=1)
    labels = csv.DictReader(open(input_csv), skipinitialspace=True).fieldnames
    solar_corr(data, labels, 'JEDI', save_png=False)


if __name__ == "__main__":
    main()





# -*- coding: utf-8 -*-

import json
import requests
from bs4 import BeautifulSoup
import random

USER_AGENTS = [
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; AcooBrowser; .NET CLR 1.1.4322; .NET CLR 2.0.50727)",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.0; Acoo Browser; SLCC1; .NET CLR 2.0.50727; Media Center PC 5.0; .NET CLR 3.0.04506)",
    "Mozilla/4.0 (compatible; MSIE 7.0; AOL 9.5; AOLBuild 4337.35; Windows NT 5.1; .NET CLR 1.1.4322; .NET CLR 2.0.50727)",
    "Mozilla/5.0 (Windows; U; MSIE 9.0; Windows NT 9.0; en-US)",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 2.0.50727; Media Center PC 6.0)",
    "Mozilla/5.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; .NET CLR 1.0.3705; .NET CLR 1.1.4322)",
    "Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.2; .NET CLR 1.1.4322; .NET CLR 2.0.50727; InfoPath.2; .NET CLR 3.0.04506.30)",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN) AppleWebKit/523.15 (KHTML, like Gecko, Safari/419.3) Arora/0.3 (Change: 287 c9dfb30)",
    "Mozilla/5.0 (X11; U; Linux; en-US) AppleWebKit/527+ (KHTML, like Gecko, Safari/419.3) Arora/0.6",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.2pre) Gecko/20070215 K-Ninja/2.1.1",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-CN; rv:1.9) Gecko/20080705 Firefox/3.0 Kapiko/3.0",
    "Mozilla/5.0 (X11; Linux i686; U;) Gecko/20070322 Kazehakase/0.4.5",
    "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.0.8) Gecko Fedora/1.9.0.8-1.fc10 Kazehakase/0.5.6",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/535.20 (KHTML, like Gecko) Chrome/19.0.1036.7 Safari/535.20",
    "Opera/9.80 (Macintosh; Intel Mac OS X 10.6.8; U; fr) Presto/2.9.168 Version/11.52",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/536.11 (KHTML, like Gecko) Chrome/20.0.1132.11 TaoBrowser/2.0 Safari/536.11",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.71 Safari/537.1 LBBROWSER",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; LBBROWSER)",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E; LBBROWSER)",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.84 Safari/535.11 LBBROWSER",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)",
    "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E; QQBrowser/7.0.3698.400)",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E)",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; Trident/4.0; SV1; QQDownload 732; .NET4.0C; .NET4.0E; 360SE)",
    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; QQDownload 732; .NET4.0C; .NET4.0E)",
    "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; .NET4.0E)",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1",
    "Mozilla/5.0 (iPad; U; CPU OS 4_2_1 like Mac OS X; zh-cn) AppleWebKit/533.17.9 (KHTML, like Gecko) Version/5.0.2 Mobile/8C148 Safari/6533.18.5",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:2.0b13pre) Gecko/20110307 Firefox/4.0b13pre",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:16.0) Gecko/20100101 Firefox/16.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11",
    "Mozilla/5.0 (X11; U; Linux x86_64; zh-CN; rv:1.9.2.10) Gecko/20100922 Ubuntu/10.10 (maverick) Firefox/3.6.10"
]


def get_header():
    return {
        'User-Agent': random.choice(USER_AGENTS),
        'Accept': 'text/html,application/xhtml+xml,'
                  'application/xml;q=0.9,*/*;q=0.8',
        'Accept-Language': 'en-US,en;q=0.5',
        'Connection': 'keep-alive',
        'Accept-Encoding': 'gzip, deflate',
    }


def get_raw_response(raw):
    """从浏览器复制访问页面的原始请求中获取数据

    :param raw: str
        从浏览器复制的访问雪球的原始请求，包含cookies等信息，如：
        raw =
            GET /stock/screener/screen.json?category=SH&exchange=&areacode=&indcode=&orderby=symbol&order=desc&current=ALL&pct=ALL&page=1&mc=0_30&eps.20180331=0.2_6.77&_=1532157090470 HTTP/1.1
            Host: xueqiu.com
            Connection: keep-alive
            Accept: application/json, text/javascript, */*; q=0.01
            cache-control: no-cache
            DNT: 1
            X-Requested-With: XMLHttpRequest
            User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36
            Referer: https://xueqiu.com/hq/screener
            Accept-Encoding: gzip, deflate, br
            Accept-Language: en-US,en;q=0.9
            Cookie: device_id=93f4ed2cb55c692c95f053da19015b52; s=fq11y14kj3; bid=e0902c871217c79df259eb5d83f46eae_j950845q; _ga=GA1.2.1448042712.1508812875; __utmz=1.1527436687.45.17.utmcsr=sogou.com|utmccn=(referral)|utmcmd=referral|utmcct=/link; __utma=1.1448042712.1508812875.1528039186.1529730232.49; remember=1; remember.sig=K4F3faYzmVuqC0iXIERCQf55g2Y; xq_a_token=6ff26751fa685eb1ceab508fcb1c6b0556ef650a; xq_a_token.sig=wSoXzThiLbzH4mXexFY2U11fUzM; xq_r_token=5a288141cd3742deacc30e09eda6d0de65501ced; xq_r_token.sig=Hp0vJJ_CRQXm_3ATbMMnyspaIBA; xq_is_login=1; xq_is_login.sig=J3LxgPVPUzbBg3Kee_PquUfih7Q; u=3987902339; u.sig=wHo1L73HiVC2HlUxrHqBNa63rVo; aliyungf_tc=AQAAAO0MKVFvPwIAyD88OjvS/B9ge3BR; Hm_lvt_1db88642e346389874251b5a1eded6e3=1531897634,1531985157,1532156465,1532157061; Hm_lpvt_1db88642e346389874251b5a1eded6e3=1532157061

    :return: response requests.Response
        原始请求对应返还的数据
    """
    raw_req = raw.strip("\n").split("\n")
    raw_req = [r.strip(" ") for r in raw_req]
    raw_req = [r for r in raw_req if r != ""]
    headers = dict()
    for r in raw_req[1:]:
        k, v = r.split(": ")
        headers[k] = v
    url = "https://" + headers['Host'] + raw_req[0].split(" ")[1]
    response = requests.get(url, headers=headers)
    return response


# 网络相关
# --------------------------------------------------------------------
def network_ready():
    """判断当前网络是否可用"""
    response = requests.get("https://wwww.baidu.com")
    if response.status_code == 200:
        return True
    else:
        return False


def get_self_ip():
    """获取本机公网ip地址"""
    TEST_IP = 'http://httpbin.org/ip'
    html = requests.get(TEST_IP)
    ip = json.loads(html.text)['origin']
    return ip


def get_ip_location(ip):
    """获取ip的地理位置信息"""
    ip = 'http://ip.chinaz.com/' + ip
    html = requests.get(ip)
    ip_info = BeautifulSoup(html.text, 'lxml')
    ip_loc = ip_info.find_all('span', {'class': 'Whwtdhalf w50-0'})
    return ip_loc[-1].text


def get_self_location():
    """获取本机网络当前地理位置"""
    ip = get_self_ip()
    location = get_ip_location(ip)
    return location





# coding: utf-8
import math
import networkx as nx
import numpy as np
from pathlib import Path
from collections import Counter

import jieba


class AttrDict(dict):
    def __init__(self, *args, **kwargs):
        super(AttrDict, self).__init__(*args, **kwargs)
        self.__dict__ = self


def sentences_similarity(sentence1, sentence2):
    """默认的用于计算两个句子相似度的函数。

    :param sentence1: list of str
        分好词的句子
    :param sentence2: list of str
        分好词的句子
    :return: float
    """
    words = list(set(sentence1 + sentence2))
    vector1 = [float(sentence1.count(word)) for word in words]
    vector2 = [float(sentence2.count(word)) for word in words]

    vector3 = [vector1[x] * vector2[x] for x in range(len(vector1))]
    vector4 = [1 for num in vector3 if num > 0.]
    co_occur_num = sum(vector4)

    if abs(co_occur_num) <= 1e-12:
        return 0.

    denominator = math.log(float(len(sentence1))) + math.log(float(len(sentence2)))  # 分母

    if abs(denominator) < 1e-12:
        return 0.

    return co_occur_num / denominator


def sort_sentences_by_pagerank(sentences,
                               sim_method=sentences_similarity,
                               pg_conf=None):
    """基于 pagerank 对句子进行排序

    :param sentences: list of str
        从 text 中得到的句子列表
    :param sim_method: function
        两个句子之间的相似度计算函数，输入为 sentence1 和 sentence2
    :param pg_conf: dict
        pagerank 算法相关参数
    :return:
    """
    if not pg_conf:
        pg_conf = {"alpha": 0.85}

    numbers = len(sentences)

    graph = np.zeros((numbers, numbers))

    for si in range(numbers):
        for sj in range(si, numbers):
            # sentence1 = list(sentences[si])
            sentence1 = jieba.cut(sentences[si])
            # sentence2 = list(sentences[sj])
            sentence2 = jieba.cut(sentences[sj])
            sim = sim_method(sentence1, sentence2)
            graph[si, sj] = sim
            graph[sj, si] = sim

    nx_graph = nx.from_numpy_matrix(graph)
    scores = nx.pagerank(nx_graph, **pg_conf)

    sorted_scores = sorted(scores.items(), key=lambda x: x[1], reverse=True)

    sentences_sorted = []
    for index, score in sorted_scores:
        item = AttrDict(index=index, sentence=sentences[index], weight=score)
        sentences_sorted.append(item)

    return sentences_sorted


def create_words_edge(words, window_size=2):
    """构造在window下的单词组合，用来构造单词之间的边。

    :param words: list
    :param window_size: int
    :return: generator
    """
    assert window_size >= 2, "window_size 必须大于等于2，否则无法构建edge"
    for x in range(1, window_size):
        if x >= len(words):
            break
        word_list2 = words[x:]
        res = zip(words, word_list2)
        for r in res:
            yield r


def sort_words_by_pagerank(segments, window_size=2, pg_conf=None):
    """使用pagerank对 words进行排序"""
    if pg_conf is None:
        pg_conf = {"alpha": 0.85}

    # 创建词表
    counter = Counter([word for sentence in segments for word in sentence])
    word2id = {w: i for i, w in enumerate(counter.keys())}
    id2word = {v: k for k, v in word2id.items()}
    words_number = len(word2id)

    # 构建图
    graph = np.zeros((words_number, words_number))

    for sentence in segments:
        for w1, w2 in create_words_edge(sentence, window_size):
            index1 = word2id[w1]
            index2 = word2id[w2]
            graph[index1][index2] = 1.0
            graph[index2][index1] = 1.0

    # 使用pagerank计算顶点权重
    nx_graph = nx.from_numpy_matrix(graph)
    scores = nx.pagerank(nx_graph, **pg_conf)
    sorted_scores = sorted(scores.items(), key=lambda x: x[1], reverse=True)

    # 输出words排序结果
    sorted_words = []
    for index, score in sorted_scores:
        item = AttrDict(word=id2word[index], weight=score)
        sorted_words.append(item)

    return sorted_words


class TextRank(object):
    def __init__(self, text,
                 sentence_sep=None,
                 seg_method=jiagu.cut,
                 stopwords=None):
        """TextRank 算法实现

        :param text: str
        :param sentence_sep: list
            句子分割符列表
        :param seg_method: function
            用于分词的函数
        :param stopwords: str or list
            如果 stopwords 是 str，则表示文件路径；否则就是 stopwords 列表。
        """
        self.text = text
        self.sentence_sep = sentence_sep
        if self.sentence_sep is None:
            self.sentence_sep = ['?', '!', ';', '？', '！', '。', '；', '……', '…']

        self.seg_method = seg_method

        if isinstance(stopwords, str):
            self.stopwords = Path(stopwords).read_text(encoding='utf-8').split('\n')
        else:
            self.stopwords = stopwords

        # 中间结果
        self.sentences = []
        self.segments = []
        self.get_segments()

    def get_segments(self):
        """文本切分： 分句 & 分词"""
        # 句子切分
        sentences = []
        sentence = []
        for _, char in enumerate(self.text):
            if char in self.sentence_sep and len(sentence) > 0:
                sentences.append(''.join(sentence))
                sentence = []
            else:
                sentence.append(char)

        # 分词
        segments = []
        for s in sentences:
            segment = self.seg_method(s)
            segments.append(segment)

        self.sentences = sentences
        self.segments = segments

    def abstract(self, n=6, sim_method=None, pg_conf=None):
        """文本摘要

        :param n: int
            选择最重要的 n 个句子作为摘要
        :param sim_method: function
            句子相似度计算函数
        :param pg_conf: dict
            pagerank 参数
        :return:
        """
        if not sim_method:
            sim_method = sentences_similarity
        if not pg_conf:
            pg_conf = {"alpha": 0.85}

        sentence_sorted = sort_sentences_by_pagerank(self.sentences,
                                                     sim_method, pg_conf)
        return sentence_sorted[:n]

    def key_words(self, n=10, window_size=2, word_min_len=1, pg_conf=None):
        """

        :param word_min_len:
        :param n:
        :param window_size:
        :param pg_conf:
        :return:
        """
        if not pg_conf:
            pg_conf = {"alpha": 0.85}
        words_sorted = sort_words_by_pagerank(self.segments, window_size, pg_conf)

        kw = []
        i = 0
        for item in words_sorted:
            if i > n:
                break
            if len(item.word) >= word_min_len:
                kw.append(item)
                i += 1

        return kw

    def key_phrases(self, kw_nums=12, min_occur_nums=2):
        key_words_set = set([item.word for item in self.key_words(n=kw_nums, word_min_len=1)])

        key_phrases_set = set()
        for sentence in self.segments:
            one = []
            for word in sentence:
                if word in key_words_set:
                    one.append(word)
                else:
                    if len(one) > 1:
                        key_phrases_set.add(''.join(one))
                    if len(one) == 0:
                        continue
                    else:
                        one = []
            # 兜底
            if len(one) > 1:
                key_phrases_set.add(''.join(one))

        return [phrase for phrase in key_phrases_set
                if self.text.count(phrase) >= min_occur_nums]













# -*- coding: utf-8 -*-

from collections import OrderedDict


class AttrDict(dict):
    """Dict that can get attribute by dot"""

    def __init__(self, *args, **kwargs):
        super(AttrDict, self).__init__(*args, **kwargs)
        self.__dict__ = self


class OrderedAttrDict(OrderedDict):
    """OrderedDict that can get attribute by dot"""

    def __init__(self, *args, **kwargs):
        super(OrderedAttrDict, self).__init__(*args, **kwargs)
        self.__dict__ = self



# -*- coding: utf-8 -*-
"""
Send Message Service
====================================================================
"""
import traceback
import requests
import os
import smtplib
from email.mime.application import MIMEApplication
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText


def server_chan_push(title, content, key=None):
    """使用server酱推送消息到微信，关于server酱，
    请参考：http://sc.ftqq.com/3.version

    :param title: str
        消息标题
    :param content: str
        消息内容，最长64Kb，可空，支持MarkDown
    :param key: str
        从[Server酱](https://sc.ftqq.com/3.version)获取的key
    :return: None
    """
    if not key:
        raise ValueError("请配置key，如果还没有key，"
                         "可以到这里申请一个：http://sc.ftqq.com/3.version")
    url = 'https://sc.ftqq.com/%s.send' % key
    requests.post(url, data={'text': title, 'desp': content})


def bear_push(title, content, send_key=None):
    """使用PushBear推送消息给所有订阅者微信，关于PushBear，
    请参考：https://pushbear.ftqq.com/admin/#/

    :param title: str
        消息标题
    :param content: str
        消息内容，最长64Kb，可空，支持MarkDown
    :param send_key: str
        从[PushBear](https://pushbear.ftqq.com/admin/#/)获取的通道send_key
    :return: None
    """
    if not send_key:
        raise ValueError("请配置通道send_key，如果还没有，"
                         "可以到这里创建通道获取：https://pushbear.ftqq.com/admin/#/")
    api = "https://pushbear.ftqq.com/sub"
    requests.post(api, data={'text': title, 'desp': content, "sendkey": send_key})


# 邮件发送器，支持发送附件。
# --------------------------------------------------------------------

class EmailSender:
    def __init__(self, from_, pw, service='qq'):
        self.from_ = from_  # 用于发送email的邮箱
        self.pw = pw  # 发送email的邮箱密码
        self.service = service
        self.smtp = smtplib.SMTP()
        self.login()

    def login(self):
        if self.service == 'qq':
            self.smtp.connect('smtp.exmail.qq.com')
            self.smtp.login(self.from_, self.pw)
        elif self.service == '163':
            self.smtp.connect('smtp.163.com')
            self.smtp.login(self.from_, self.pw)
        else:
            raise ValueError('目前仅支持163和qq邮箱！')

    def _construct_msg(self, to, subject, content, files=None):
        """构造email信息

        parameters
        ---------------
        subject     邮件主题
        content     邮件文本内容
        files       附件（list）
                    可以是相对路径下的文件，也可以是绝对路径下的文件；
                    推荐使用绝对路径。

        return
        --------------
        msg         构造好的邮件信息
        """
        msg = MIMEMultipart()
        msg['from'] = self.from_
        msg['to'] = to
        msg['subject'] = subject
        txt = MIMEText(content)
        msg.attach(txt)

        # 添加附件
        if files is not None:
            for file in files:
                f = MIMEApplication(open(file, 'rb').read())
                f.add_header('Content-Disposition', 'attachment',
                             filename=os.path.split(file)[-1])
                msg.attach(f)

        return msg

    def send_email(self, to, subject, content, files=None):
        """登录邮箱，发送msg到指定联系人

        :param to: str: 收件人邮箱
        :param subject: str: 主题
        :param content: str: 内容
        :param files:  list: 附件列表
        :return: None
        """
        msg = EmailSender._construct_msg(self, to, subject, content, files=files)
        try:
            self.smtp.sendmail(self.from_, to, str(msg))
        except:
            traceback.print_exc()
            self.login()
            self.smtp.sendmail(self.from_, to, str(msg))

    def quit(self):
        self.smtp.quit()  # 退出登录


def send_email(sender, pw, to, subject, content, files=None, service='163'):
    """send email, recommended use 163 mailbox service, as it is tested.

    :param sender: str
        email address of sender
    :param pw: str
        password for sender
    :param to: str
        email addressee
    :param subject: str
        subject of email
    :param content: str
        content of email
    :param files: list
        path list of attachments
    :param service: str
        smtp server address, optional is ['163', 'qq']
    :return: None
    """
    se = EmailSender(from_=sender, pw=pw, service=service)
    se.send_email(to=to, subject=subject, content=content, files=files)
    se.quit()



# -*- coding: utf-8 -*-

import os
import PyPDF2


def merge_pdf(pdf_files, output_file):
    """合并多个pdf文件

    :param pdf_files: list
        需要合并的pdf文件列表
    :param output_file: str
        合并后的pdf文件路径
    """
    merger = PyPDF2.PdfFileMerger()
    for file in pdf_files:
        f = open(file, 'rb')
        file_rd = PyPDF2.PdfFileReader(f)
        # use file name as bookmark
        bookmark = os.path.basename(os.path.splitext(file)[0])
        if file_rd.isEncrypted:
            print('%s is encrypted, not supported!' % file)
            continue
        merger.append(file_rd, bookmark=bookmark, import_bookmarks=True)
        f.close()
    merger.write(output_file)
    merger.close()




# -*- coding: utf-8 -*-

import numpy as np


def euclidean_distant(vector1, vector2):
    """欧式距离"""
    vector1 = np.mat(vector1)
    vector2 = np.mat(vector2)
    return np.sqrt((vector1 - vector2) * (vector1 - vector2).T).item()


def manhattan_distant(vector1, vector2):
    """曼哈顿距离"""
    vector1 = np.mat(vector1)
    vector2 = np.mat(vector2)
    return np.sum(np.abs(vector1 - vector2))


def cosine_distant(vector1, vector2):
    """余弦距离"""
    vector1 = np.mat(vector1)
    vector2 = np.mat(vector2)
    vector1_norm = np.linalg.norm(vector1)
    vector2_norm = np.linalg.norm(vector2)
    dot_norm = vector1_norm * vector2_norm
    dot_vs = np.dot(vector1, vector2)
    return np.divide(dot_vs, dot_norm).item()


def pearson_similar(vector1, vector2):
    """皮尔逊相关系数"""
    if len(vector1) < 3:
        return 1.0
    return 0.5 + 0.5 * np.corrcoef(vector1, vector2, rowvar=0)[0][1]



# -*- coding: utf-8 -*-
"""

Reference:
1. https://github.com/jasonwbw/DensityPeakCluster
2. Rodriguez, A. & Laio, A. Clustering by fast search and
   find of density peaks. Science 344, 1492–1496 (2014).
====================================================================
"""
from copy import deepcopy
import numpy as np
import matplotlib
import matplotlib.pyplot as plt

matplotlib.rcParams['font.size'] = 18


class FastCluster:
    def __init__(self, points, distant_func, dc=None):
        """Implement Algorithm demonstrated on [1].

        [1]. Rodriguez, A. & Laio, A. Clustering by fast search and
        find of density peaks. Science 344, 1492–1496 (2014).
        """
        # distant_func 的输入是 point1，point2
        self.distant_func = distant_func
        self.dc = dc
        self.points = points

        self._distant_matrix(points)
        if self.dc is None:
            self.dc = self._default_dc()

        self._local_density()

        # 重要中间结果
        self.centers = None
        self.clusters = None

    def _default_dc(self):
        dist = deepcopy(self.dist_matrix)
        dist = dist.reshape(-1)
        dist.sort()
        return round(dist[int(len(dist) * 0.02)], 4)

    def _distant_matrix(self, points):
        """计算全部样本的距离矩阵

        :param points: list of sample
            样本点
        :return:
        """
        self.total_number = total_number = len(points)
        dist = np.zeros((total_number, total_number))

        for i in range(total_number):
            for j in range(i+1, total_number):
                d = self.distant_func(points[i], points[j])
                dist[i][j] = d
                dist[j][i] = d
        self.dist_matrix = dist

    def _local_density(self):
        if self.dist_matrix is None:
            raise ValueError('distant matrix is None')

        total_number = self.total_number

        # 计算局部密度
        ld = np.zeros(total_number)
        for i in range(total_number):
            ld[i] = np.sum(np.exp(-(self.dist_matrix[i]/self.dc)**2))
            # row = self.dist_matrix[i]
            # ld[i] = np.sum(row <= self.dc)
        self.ld = ld

        # 计算最小距离
        md = np.zeros(total_number)
        for i in range(total_number):
            row = self.dist_matrix[i][ld > ld[i]]
            if len(row) > 0:
                md[i] = np.min(row)
            else:
                md[i] = np.max(self.dist_matrix[i])
        self.md = md
        self.gamma = ld * md

    def plot_decision_graph(self):
        fig = plt.figure(figsize=(16, 9))
        ax = fig.subplots(1, 1)
        y = sorted(self.gamma, reverse=True)
        x = list(range(len(y)))
        ax.scatter(x, y)
        plt.xlabel('n'), plt.ylabel('gamma')
        plt.title("decision graph of fast cluster")

    def _nearest_neighbor(self, clusters, index):
        dist_matrix = self.dist_matrix
        ld = self.ld
        dd = 10000000
        neighbor = -1.0
        for i in range(self.total_number):
            if dist_matrix[index, i] < dd and ld[index] < ld[i]:
                dd = dist_matrix[index, i]
                neighbor = i
        if clusters[neighbor] == -1.0:
            clusters[neighbor] = self._nearest_neighbor(clusters, neighbor)
        return clusters[neighbor]

    def fit(self, rate):
        gamma = self.gamma
        threshold = rate * np.max(gamma)
        # 找出类簇中心
        clusters = -1 * np.ones(self.total_number, dtype=np.int)
        center = 0
        for i in range(self.total_number):
            if gamma[i] > threshold:
                clusters[i] = center
                center += 1

        for i in range(self.total_number):
            if clusters[i] == -1:
                clusters[i] = self._nearest_neighbor(clusters, i)

        self.clusters = clusters
        return clusters



# -*- coding: utf-8 -*-

"""
use pynput to monitor & control keyboard

Ref: https://pynput.readthedocs.io/en/latest/keyboard.html
====================================================================
"""

from pynput.keyboard import Key, Listener, Controller


class KeyboardMonitor:
    def __init__(self, name=None):
        if name:
            self.name = name
        else:
            self.name = "KEYBOARD MONITOR"

    @staticmethod
    def on_press(key):
        try:
            print('alphanumeric key {0} pressed'.format(
                key.char))
        except AttributeError:
            print('special key {0} pressed'.format(
                key))

    @staticmethod
    def on_release(key):
        print('{0} released'.format(
            key))
        if key == Key.esc:
            # Stop listener
            return False

    def run(self):
        with Listener(on_press=self.on_press,
                      on_release=self.on_release) as listener:
            listener.join()


KeyboardController = Controller


def run_keyboard_monitor():
    km = KeyboardMonitor()
    km.run()




# -*- coding: utf-8 -*-
import os
import pickle
import shutil
import zipfile
from threading import Lock

lock = Lock()


def legitimize(text):
    """Converts a string to a valid filename.
    """
    import platform
    os_ = platform.system()
    # POSIX systems
    text = text.translate({
        0: None,
        ord('/'): '-',
        ord('|'): '-',
    })

    if os_ == 'Windows':
        # Windows (non-POSIX namespace)
        text = text.translate({
            # Reserved in Windows VFAT and NTFS
            ord(':'): '-',
            ord('*'): '-',
            ord('?'): '-',
            ord('\\'): '-',
            ord('\"'): '\'',
            # Reserved in Windows VFAT
            ord('+'): '-',
            ord('<'): '-',
            ord('>'): '-',
            ord('['): '(',
            ord(']'): ')',
        })
    else:
        # *nix
        if os_ == 'Darwin':
            # Mac OS HFS+
            text = text.translate({
                ord(':'): '-',
            })

        # Remove leading .
        if text.startswith("."):
            text = text[1:]

    text = text[:82]  # Trim to 82 Unicode characters long
    return text


def modify_content(file, old, new):
    """替换文件中的指定内容，逐行进行"""
    try:
        lines = open(file, 'r').readlines()
        f_len = len(lines) - 1
        for i in range(f_len):
            if old in lines[i]:
                lines[i] = lines[i].replace(old, new)
        open(file, 'w').writelines(lines)
    except Exception as e:
        print(e)


def batch_del_file(path, in_name=None, yn=None):
    """批量删除文件（给出一定的规则）"""
    if in_name is None:
        file_to_del = [os.path.join(path, i) for i in os.listdir(path)]
    else:
        file_to_del = [os.path.join(path, i) for i in os.listdir(path) if in_name in i]
    print('以下是要删除的文件列表：\n')
    print(file_to_del)

    if yn is None:
        yn = input('删除以上文件（输入：y/n）, 保留部分文件（输入：k）：')

    if yn == 'k':
        file_to_save = []
        while True:
            f = input('请输入要保留的文件名（输入“b”结束）：')
            if f == 'b':
                break
            file_to_save.append(f)
        file_to_del = [i for i in file_to_del if i not in file_to_save]
        for file in file_to_del:
            os.remove(file)
            print('已删除：', file)
    elif yn == 'y' or yn == 'yes':
        for file in file_to_del:
            os.remove(file)
            print('已删除：', file)
    else:
        print('没有任何文件被删除！')


def batch_move_file(path, dest, in_name=None, copy=True):
    """将path目录下文件名中包含的in_name的文件移动到dest目录下

    parameters
    ------------
    path        需要转移文件的目录
    dest        目标目录
    in_name     str, 默认是None。文件名过滤规则

    example
    ------------
    # 将path目录下的所有txt文件转移到dest目录
    move_file(path, dest, in_name='.txt')
    默认会覆盖目标目录下的同名文件
    """
    assert os.path.exists(path), '%s 文件夹不存在' % path
    assert os.path.exists(dest), '%s 文件夹不存在' % dest
    if copy:
        mv = shutil.copy
    else:
        mv = shutil.move

    if in_name is None:
        file_list = os.listdir(path)
    else:
        file_list = [i for i in os.listdir(path) if in_name in i]

    if len(file_list) != 0:
        for file in file_list:
            f = os.path.join(path, file)
            d = os.path.join(dest, file)
            mv(f, d)
            print('已转移：%s' % f)
    else:
        if in_name is None:
            print('%s 中没有文件' % path)
        else:
            print('%s 中没有包含【 %s 】的文件' % (path, in_name))


def batch_rename_file(path, f, t):
    """根据replaces中定义的规则，批量重命名"""
    files = os.listdir(path)
    for file in files:
        if f in file:
            new_fn = file.replace(f, t)
            old = os.path.join(path, file)
            new = os.path.join(path, new_fn)
            os.rename(old, new)


def make_zip(folder_path, output_filename):
    """将目录中除zip之外的文件打包成zip文件(包括子文件夹)
    空文件夹不会被打包

    example
    ----------------
    make_zip('results','zips//招标信息结果_2017-05-09.zip')
    """
    cwd = os.getcwd()

    # 获取需要打包的文件列表
    file_lists = []
    for root, dirs, files in os.walk(folder_path):
        for file in files:
            file_1 = os.path.join(root, file).replace(folder_path + '/', '')
            if 'zip' not in file_1:
                file_lists.append(file_1)

    # 将文件列表打包成zip
    os.chdir(folder_path)
    with zipfile.ZipFile(output_filename, 'w') as myzip:
        for file in file_lists:
            myzip.write(file)

    # 将工作目录切换回原始
    os.chdir(cwd)


# write, read, empty
# --------------------------------------------------------------------

def write_file(file, content=None, mode="a", encoding='utf-8'):
    lock.acquire(blocking=True, timeout=30)
    with open(file, mode, encoding=encoding) as f:
        if isinstance(content, str):
            content += "\n"
            f.write(content)
        elif isinstance(content, list):
            content = [i.strip("\n") + '\n' for i in content]
            f.writelines(content)
        elif content is None:
            pass
        else:
            raise ValueError("If content is not None, it must be list or str!")
    lock.release()


def read_file(file, mode="r", encoding='utf-8'):
    lock.acquire(blocking=True, timeout=30)
    with open(file, mode, encoding=encoding) as f:
        lines = f.readlines()
        lines = [line.strip("\n") for line in lines]
    lock.release()
    if len(lines) > 0:
        return lines
    else:
        raise ValueError("file is empty!")


def empty_file(file, mode="w", encoding='utf-8'):
    """empty file"""
    with open(file, mode, encoding=encoding) as f:
        f.truncate()


# pickle
# --------------------------------------------------------------------
def save_to_pkl(file, data, protocol=None):
    if protocol is None:
        protocol = pickle.HIGHEST_PROTOCOL
    with open(file, 'wb') as f:
        pickle.dump(data, f, protocol=protocol)


def read_from_pkl(file):
    with open(file, 'rb') as f:
        data = pickle.load(f)
    return data



# -*- coding: utf-8 -*-

from PIL import Image
import os


def img2pdf(path, pdf_name=None):
    """合并path目录下的所有图片到pdf文件中

    :param path: str
        图片路径
    :param pdf_name: str
        默认值 图片路径.pdf; pdf文件名
    """
    print("merge images in %s ..." % path)
    if not pdf_name:
        pdf_name = os.path.join(path, '%s.pdf' % os.path.split(path)[1])
    file_list = os.listdir(path)

    # filter pic in path
    pic_names = []
    img_suffix = ['.png', '.jpg', '.jpeg']
    for x in file_list:
        _, suffix = os.path.splitext(x)
        if suffix.lower() in img_suffix:
            pic_names.append(os.path.join(path, x))
    pic_names.sort()

    if len(pic_names) == 0:
        print("There is no images in %s" % path)
        return

    im_list = []
    im1 = Image.open(pic_names[0])
    pic_names.pop(0)
    for i in pic_names:
        img = Image.open(i)
        if img.mode == "RGBA":
            img = img.convert('RGB')
            im_list.append(img)
        else:
            im_list.append(img)
    im1.save(pdf_name, "PDF", resolution=100.0, save_all=True,
             append_images=im_list)
    print("images saved to：", pdf_name)



# -*- coding: utf-8 -*-

import pandas as pd


def count_nans(df):
    """
    Count the total number of NaNs in every column

    Parameters
    --------------------
        df    pd.DataFrame

    Returns
    --------------------
        nas_df    pd.DataFrame

    """
    cols = df.columns
    res = []
    for col in cols:
        length = len(df[col])
        not_nas = len(df[col].dropna())
        nas = length - not_nas
        rate = round(nas/length, 4)
        # add unique value
        uv = len(df[col].unique())
        res_ = (col, nas, not_nas, rate, uv)
        res.append(res_)
    nas_df = pd.DataFrame(res, columns=['Column', 'NaNs', 'Not_NaNs',
                                        'Rate', 'UV'])
    return nas_df




# -*- coding: utf-8 -*-
"""
IP代理自动获取，并检查可用性
====================================================================
"""

import requests
from urllib.error import HTTPError


# 测试代理是否可用
# --------------------------------------------------------------------

def is_usable(host, port, timeout=3):
    """测试代理是否可用

    params
    ----------
    host        ip地址
    port        端口号
    timeout     默认值为3，通过设置这个参数可以过滤掉一些速度慢的代理

    example
    ----------
    is_usable('222.180.24.13', '808', timeout=3)

    """
    try:
        proxies = {
            'http': 'http://%s:%s' % (host, port),
            'https': 'https://%s:%s' % (host, port)
        }
        requests.get('http://www.baidu.com/', proxies=proxies, timeout=timeout)
    except HTTPError:
        print('failed: ', host, port)
        return False
    else:
        print('success: ', host, port)
        return True





def quick_sort(array, left, right):
    """快速排序"""
    if left >= right:
        return

    low = left
    high = right
    key = array[low]

    while left < right:

        # 将大于key的值放到右边
        while left < right and array[right] > key:
            right -= 1
        array[left] = array[right]

        # 将小于key的值放到左边
        while left < right and array[left] <= key:
            left += 1
        array[right] = array[left]

    # 把key放到对应的位置
    array[right] = key

    # 递归执行（左右同时排序）
    quick_sort(array, low, left - 1)
    quick_sort(array, left + 1, high)






from setuptools import setup, find_packages
import zb

setup(
    name="zb",
    version=zb.__version__,
    keywords=("zb", "tool", "algorithm"),
    description="Personal Package of ZengBin",
    long_description="Personal tools and algorithms.",
    license="Apache-2.0",

    url="https://github.com/zengbin93/zb",
    author=zb.__author__,
    author_email="zeng_bin8888@163.com",

    packages=find_packages(exclude=['test', 'images']),
    include_package_data=True,
    install_requires=[
        "requests", "pandas", "click", "numpy", 'bs4', 'jieba',
        "pypdf2", "pynput"
    ],
    package_data={'': ['*.csv', '*.txt']},
    entry_points={
        "console_scripts": [
            "zb=zb.cli:zb"
        ]
    },
    classifiers=[
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
        ]
)



# -*- coding: utf-8 -*-
"""
心食谱 爬虫

https://www.xinshipu.com/
====================================================================
"""

import requests
from bs4 import BeautifulSoup
import re
from urllib.parse import urljoin

from zb.crawlers.utils import get_header

HOME_URL = "https://www.xinshipu.com/"


def get_recipe_detail(recipe_url):
    """从url中获取菜谱详细信息

    :param recipe_url: str
        菜谱url，如：https://www.xinshipu.com/zuofa/598775;
        https://www.xinshipu.com//zuofa/749342
    :return:dict
    """
    response = requests.get(recipe_url, headers=get_header())
    html = BeautifulSoup(response.text, 'lxml')

    # 获取菜名
    name = html.find("div", {"class": "re-up"}).h1.text

    # 主图
    img = html.find("div", {"class": "gallery"}).a['href']
    img = urljoin(HOME_URL, img)

    all_info = html.find_all("div", {"class": "dd"})

    if len(all_info) == 4:
        # 简介
        intro = re.sub('\n|\t|\r| ', '', all_info[0].text)
        material_i = 1
        method_i = 2
    else:
        intro = None
        material_i = 0
        method_i = 1

    # 食材
    material = all_info[material_i].text.strip()
    material = re.sub('\r\n|\r\n \n|\n\n\n', '\n', material)

    # 做法
    try:
        method_steps = html.find("ol", {"class": "re-step-wpic"}).find_all('li')
        method = []
        for i, m in enumerate(method_steps, 1):
            step = dict(step_num=i)
            step['text'] = m.text.strip()
            if m.img:
                step['img_url'] = urljoin(HOME_URL, m.img['src'])
            method.append(step)
    except:
        method = all_info[method_i].text.strip()
        method = re.sub('\r\n|\r\n \n|\n\n\n\n', '\n', method)

    # 相关菜品
    classify = all_info[-1].text.strip()
    if '\xa0\xa0' in classify:
        classify = classify.replace('\xa0\xa0', ' | ')
    else:
        classify = ""

    return {
        "name": name,
        "url": recipe_url,
        "img": img,
        "intro": intro,
        "material": material,
        "method": method,
        "classify": classify
    }


def get_all_classify():
    """获取全部菜谱分类"""
    url = "https://www.xinshipu.com/%E8%8F%9C%E8%B0%B1%E5%A4%A7%E5%85%A8.html"
    response = requests.get(url, headers=get_header())
    html = BeautifulSoup(response.text, "lxml")

    all_a = html.find("div", {'class': "detail-cate-list clearfix mt20"}).find_all('a')
    classify = dict()

    for a in all_a:
        if a.has_attr('rel') and not a.has_attr('class'):
            class_url = urljoin(HOME_URL, a['href'])
            classify[a.text] = class_url

    return classify


def get_class_recipes(class_url, max_page=20, sleep=0.1):
    """获取某个菜谱分类url下的所有菜谱url"""
    class_url = class_url + "?page={page}"
    recipes = dict()

    # 暴力爬取方案，每个菜谱分类请求100页
    for page in range(1, max_page):
        time.sleep(sleep)
        url = class_url.format(page=page)
        print("current url: ", url)
        response = requests.get(url, headers=get_header())
        html = BeautifulSoup(response.text, "lxml")

        # 获取本页菜谱
        menus = html.find("div", {'class': 'new-menu-list search-menu-list clearfix mt10'})
        if menus:
            print("get recipes fail: ", url)
            menus = menus.find_all('a')
            for m in menus:
                name = re.sub("\n| ", "", m.text)
                recipe_url = urljoin(HOME_URL, m['href'])
                recipes[name] = recipe_url

        # 判断是否是最后一页
        # next_page = html.find('div', {'class': 'paging mt20'}).text
        # if "下一页" in next_page:
        #     page += 1
        # else:
        #     break
    return recipes


if __name__ == '__main__':
    import json
    import os
    import time

    data_path = r"C:\ZB\git_repo\recipes"

    # 分类url
    print("获取分类url ...")
    file_classify = os.path.join(data_path, "classify_url.json")
    if os.path.exists(file_classify):
        classify = json.load(open(file_classify, 'r', encoding='utf-8'))
    else:
        classify = get_all_classify()
        json.dump(classify, open(file_classify, 'w', encoding='utf-8'),
                  indent=2, ensure_ascii=False)

    # 食谱url
    print("获取食谱url ...")
    file_recipes = os.path.join(data_path, "recipes_url.json")
    if os.path.exists(file_recipes):
        recipes_url = json.load(open(file_recipes, 'r', encoding='utf-8'))
    else:
        recipes_url = dict()
        for class_url in classify.values():
            try:
                class_recipes = get_class_recipes(class_url)
                recipes_url.update(class_recipes)
                print("current total recipes number is ", len(recipes_url))
                json.dump(recipes_url, open(file_recipes, 'w', encoding='utf-8'),
                          indent=2, ensure_ascii=False)
            except:
                continue

    # 食谱详细信息
    print("获取食谱详细信息 ...")
    file_recipes_detail = os.path.join(data_path, "recipes_detail.json")
    if os.path.exists(file_recipes_detail):
        recipes_detail = json.load(open(file_recipes_detail, 'r', encoding='utf-8'))
    else:
        recipes_detail = dict()
    for recipe_url in recipes_url.values():
        time.sleep(0.1)
        if recipe_url in recipes_detail.keys():
            continue
        try:
            print("current: ", recipe_url)
            detail = get_recipe_detail(recipe_url)
            recipes_detail[recipe_url] = detail
            json.dump(recipes_detail, open(file_recipes_detail, 'w', encoding='utf-8'),
                      indent=2, ensure_ascii=False)
        except:
            print("fail: ", recipe_url)
            continue










# -*- coding: utf-8 -*-

"""
use pynput to monitor & control mouse

Ref: https://pynput.readthedocs.io/en/latest/mouse.html
====================================================================
"""


from pynput.mouse import Controller, Listener


class MouseMonitor:
    def __init__(self, name=None):
        if name:
            self.name = name
        else:
            self.name = "MOUSE MONITOR"

    @staticmethod
    def on_move(x, y):
        print('Pointer moved to {0}'.format(
            (x, y)))

    @staticmethod
    def on_click(x, y, button, pressed):
        print('{0} at {1}'.format(
            'Pressed' if pressed else 'Released',
            (x, y)))
        if not pressed:
            # Stop listener
            return False

    @staticmethod
    def on_scroll(x, y, dx, dy):
        print('Scrolled {0} at {1}'.format(
            'down' if dy < 0 else 'up',
            (x, y)))

    def run(self):
        # Collect events until released
        with Listener(on_move=self.on_move, on_click=self.on_click,
                      on_scroll=self.on_scroll) as listener:
            listener.join()


MouseController = Controller

